/***************************************************************************
                          ax25helper.c  -  description
                             -------------------
    begin                : Mon Apr 12 2004
    copyright            : (C) 2004 by Marco Savegnago IW3FQG
    email                : msave@tin.it

  SVClick is simple program (the my first one on linux...)
  that download camera photo from supervozelj node using
  ax25 stream and convert it to jpeg format.

  Part of the code was from:

  svphoto - Read SuperVozelj photo, and write it in a readable
    jpeg or png format
    Copyright (C) 2001  Paolo Subiaco

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "svclick.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <sys/time.h>
#include <string.h>

#ifdef HAVE_NETAX25_AX25_H
#include <netax25/ax25.h>
#else
#include <netax25/kernel_ax25.h>
#endif
#ifdef HAVE_NETROM_NETROM_H
#include <netrom/netrom.h>
#else
#include <netax25/kernel_netrom.h>
#endif
#ifdef HAVE_NETROSE_ROSE_H
#include <netrose/rose.h>
#else
#include <netax25/kernel_rose.h>
#endif
#include <sys/socket.h>

#include <netax25/axlib.h>
#include <netax25/axconfig.h>
#include <netax25/nrconfig.h>
#include <netax25/rsconfig.h>


int ax25mode = -1;
int backoff = -1;
int paclen;
int window;

int AX25CheckConfig(char* csz_Port)
{
  if (ax25_config_load_ports() == 0)
    return  1;
  //  Read maxframe and paclen parameters

  window = ax25_config_get_window(csz_Port);
  paclen = ax25_config_get_paclen(csz_Port);

  return  0;
}

int AX25SetParameters(int i_AX25Connection)
{
  if (setsockopt(i_AX25Connection, SOL_AX25, AX25_WINDOW, &window, sizeof(window)) == -1)
  {
    perror("AX25_WINDOW");
    close(i_AX25Connection);
    return (-1);
  }

  if (setsockopt(i_AX25Connection, SOL_AX25, AX25_PACLEN, &paclen, sizeof(paclen)) == -1)
  {
    perror("AX25_PACLEN");
    close(i_AX25Connection);
    return (-1);
  }

  if (backoff != -1)
  {
    if (setsockopt(i_AX25Connection, SOL_AX25, AX25_BACKOFF, &backoff, sizeof(backoff)) == -1)
    {
      perror("AX25_BACKOFF");
      close(i_AX25Connection);
      return (-1);
    }
  }

  if (ax25mode != -1)
  {
    if (setsockopt(i_AX25Connection, SOL_AX25, AX25_EXTSEQ, &ax25mode, sizeof(ax25mode)) == -1)
    {
      perror("AX25_EXTSEQ");
      close(i_AX25Connection);
      return (-1);
    }
  }

  return  0;
}

int AX25ConnectTo(char* csz_Port, const  char *csz_Address)
{
  int addrlen = 0;
  int i_AX25Connection;

  union
  {
    struct full_sockaddr_ax25 ax25;
    struct sockaddr_rose rose;
  }
  sockaddr;

  if (csz_Address == NULL)
  {
    fprintf(stderr, "call: too few arguments for AX.25\n");
    return (-1);
  }

  if ((i_AX25Connection = socket(AF_AX25, SOCK_SEQPACKET, 0)) < 0)
  {
    perror("socket");
    return (-1);
  }

  ax25_aton(ax25_config_get_addr(csz_Port), &sockaddr.ax25);

  sockaddr.ax25.fsa_ax25.sax25_family = AF_AX25;
  addrlen = sizeof(struct full_sockaddr_ax25);

  if(AX25SetParameters(i_AX25Connection) == -1)
  {
    puts("Errore funzione AX25SetParameters");
    return (-1);
  }

  if (bind(i_AX25Connection, (struct sockaddr *) &sockaddr, addrlen) == -1)
  {
    perror("bind");
    close(i_AX25Connection);
    return (-1);
  }

  if (ax25_aton((const char*)csz_Address, &sockaddr.ax25) == -1)
  {
    close(i_AX25Connection);
    fprintf(stderr, "Error on ax25_aton_arglist");
    return (-1);
  }

  addrlen = sizeof(struct full_sockaddr_ax25);

  if (connect(i_AX25Connection, (struct sockaddr *) &sockaddr, addrlen))
  {
    printf("\n");
    perror("connect");
    close(i_AX25Connection);
    return (-1);
  }

  return (i_AX25Connection);
}

int AX25WaitString(int i_AX25Connection, const char* psz_SearchString, char* psz_Line, int i_Timeout)
{
  time_t t_watchdog;
  int i_bufferLength;
  int i_bufferCount;
  byte  by_buffer[MAXREAD];

  int i=0;

  psz_Line[0]=0;

  t_watchdog=time(NULL);

  while (1)
  {
    usleep(500000);

    while ((i_bufferLength=read(i_AX25Connection,by_buffer,MAXREAD-1))>0)
    {
      // parse the chars arrived
      for (i_bufferCount=0;i_bufferCount<i_bufferLength;i++,i_bufferCount++)
      {
        psz_Line[i]=by_buffer[i_bufferCount];

        if (by_buffer[i_bufferCount]=='\r' || by_buffer[i_bufferCount]=='\n' || i==MAXREAD-1)
        {
          // new line!! parse this line.
          psz_Line[i]=0;

          if (strstr(psz_Line,psz_SearchString))
          {
            i_bufferCount++;
            return 1; // line found: return true!
          }

          i=-1;
        }
      }
    }

    if ((time(NULL)-t_watchdog)>i_Timeout)
      return 0;
  }
}
